import { Meta } from '@storybook/react';

import { configureStore } from '@reduxjs/toolkit';
import { Provider } from 'react-redux';
import {
  big2ndChamberQueryResult,
  bigMockQueryResults,
  big2ndChamberSchemaRaw,
  smallFlightsQueryResults,
  mockLargeQueryResults,
  recommendationPersonActedInMovieQueryResult,
  recommendationPersonActedInMovieQueryResultPayload,
  slackReactionToThreadedMessageQueryResultPayload,
} from '../../../mock-data';

// export * from './mockLargeQueryResults';
// export * from './big2ndChamberQueryResult';
// export * from './bigMockQueryResults';
// export * from './mockQueryResults';
// export * from './smallFlightsQueryResults';
// export * from './mockMobilityQueryResult';
// export * from './typesMockQueryResults';

import { VisualizationPanel } from '../../panel/visualization';
import {
  assignNewGraphQueryResult,
  graphQueryResultSlice,
  querybuilderSlice,
  schemaSlice,
  setSchema,
  visualizationSlice,
} from '../../../data-access/store';

import { SchemaUtils } from '../../../schema/schema-utils';
import { simpleSchemaAirportRaw } from '../../../mock-data/schema/simpleAirportRaw';
import { Visualizations, setActiveVisualization } from '@graphpolaris/shared/lib/data-access/store/visualizationSlice';

const Component: Meta<typeof VisualizationPanel> = {
  title: 'Visualizations/MatrixVis',
  component: VisualizationPanel,
  decorators: [
    (story) => (
      <Provider store={Mockstore}>
        <div
          style={{
            width: '100%',
            height: '100vh',
          }}
        >
          {story()}
        </div>
      </Provider>
    ),
  ],
};

const Mockstore = configureStore({
  reducer: {
    schema: schemaSlice.reducer,
    graphQueryResult: graphQueryResultSlice.reducer,
    visualize: visualizationSlice.reducer,
    querybuilder: querybuilderSlice.reducer,
  },
});

export const TestWithData = {
  layout: 'fullscreen',
  args: {
    loading: false,
  },
  play: async () => {
    const dispatch = Mockstore.dispatch;
    const schema = SchemaUtils.schemaBackend2Graphology(simpleSchemaAirportRaw);
    dispatch(setSchema(schema.export()));
    dispatch(
      assignNewGraphQueryResult({
        queryID: '1',
        result: {
          type: 'nodelink',
          payload: {
            nodes: [
              { id: 'agent/007', attributes: { name: 'Daniel Craig' } },
              { id: 'villain', attributes: { name: 'Le Chiffre' } },
            ],
            edges: [{ id: 'escape/escape', from: 'agent/007', to: 'villain', attributes: { name: 'Escape' } }],
          },
        },
      })
    );
    dispatch(setActiveVisualization(Visualizations.Matrix));
  },
};

export const TestWithNoData = {
  args: { loading: false },
  play: async () => {
    const dispatch = Mockstore.dispatch;
    dispatch(
      assignNewGraphQueryResult({
        queryID: '1',
        result: {
          type: 'nodelink',
          payload: {
            nodes: [],
            edges: [],
          },
        },
      })
    );
    dispatch(setActiveVisualization(Visualizations.Matrix));
  },
};

export const TestWithBig2ndChamber = {
  args: { loading: false },
  play: async () => {
    const dispatch = Mockstore.dispatch;
    dispatch(assignNewGraphQueryResult({ queryID: '1', result: { type: 'nodelink', payload: big2ndChamberQueryResult } }));
    dispatch(setActiveVisualization(Visualizations.Matrix));
  },
};

export const TestWithSmallFlights = {
  args: { loading: false },
  play: async () => {
    const dispatch = Mockstore.dispatch;
    dispatch(assignNewGraphQueryResult({ queryID: '1', result: { type: 'nodelink', payload: smallFlightsQueryResults } }));
    dispatch(setActiveVisualization(Visualizations.Matrix));
  },
};

export const TestWithLargeQueryResult = {
  args: { loading: false },
  play: async () => {
    const dispatch = Mockstore.dispatch;
    dispatch(assignNewGraphQueryResult({ queryID: '1', result: { type: 'nodelink', payload: mockLargeQueryResults } }));
    dispatch(setActiveVisualization(Visualizations.Matrix));
  },
};

export const TestWithRecommendationPersonActedInMovieQueryResult = {
  args: { loading: false },
  play: async () => {
    const dispatch = Mockstore.dispatch;
    dispatch(assignNewGraphQueryResult(recommendationPersonActedInMovieQueryResultPayload));
    dispatch(setActiveVisualization(Visualizations.Matrix));
  },
};

export const TestWithSlackReactionToThreadedMessageQueryResult = {
  args: { loading: false },
  play: async () => {
    const dispatch = Mockstore.dispatch;
    dispatch(assignNewGraphQueryResult(slackReactionToThreadedMessageQueryResultPayload));
    dispatch(setActiveVisualization(Visualizations.Matrix));
  },
};

export default Component;
